@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -55,9 +55,16 @@ class timer {
 
   timer               round_seconds() const              { return (m_time / 1000000) * 1000000; }
 
-  timeval             tval() const                       { return (timeval) { m_time / 1000000, m_time % 1000000}; }
+  timeval             tval() const {
+    timeval val;
+    val.tv_sec  = m_time / 1000000;
+    val.tv_usec = m_time % 1000000;
+    return val;
+  }
 
   static timer        current();
+  static int64_t      current_seconds()                   { return current().seconds(); }
+  static int64_t      current_usec()                      { return current().usec(); }
   static timer        from_seconds(uint32_t seconds)      { return rak::timer((uint64_t)seconds * 1000000); }
   static timer        from_milliseconds(uint32_t seconds) { return rak::timer((uint64_t)seconds * 1000); }
 
