@@ -36,6 +36,7 @@
 
 #include "config.h"
 
+#include <ctime>
 #include <rak/functional.h>
 #include <rak/functional_fun.h>
 #include <sigc++/adaptors/bind.h>
@@ -393,6 +394,28 @@ cmd_view_size_not_visible(__UNUSED rpc::target_type target, const torrent::Objec
   return (*control->view_manager()->find_throw(rawArgs.as_string()))->size_not_visible();
 }
 
+torrent::Object
+cmd_view_persistent(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
+  const std::string& args = rawArgs.as_string();
+  core::View* view = *control->view_manager()->find_throw(args);
+  
+  if (!view->get_filter().empty() || !view->get_event_added().empty() || !view->get_event_removed().empty())
+    throw torrent::input_error("Cannot set modified views as persitent.");
+
+  view->set_filter("d.views.has=" + args);
+  view->set_event_added("d.views.push_back_unique=" + args);
+  view->set_event_removed("d.views.remove=" + args);
+
+  return torrent::Object();
+}
+
+torrent::Object
+cmd_ui_set_view(__UNUSED rpc::target_type target, const torrent::Object& rawArgs) {
+  control->ui()->download_list()->set_current_view(rawArgs.as_string());
+
+  return torrent::Object();
+}
+
 torrent::Object
 cmd_ui_unfocus_download(core::Download* download, const torrent::Object& rawArgs) {
   control->ui()->download_list()->unfocus_download(download);
@@ -443,6 +466,8 @@ initialize_command_ui() {
 
   CMD_G_STRING("view.size",              rak::ptr_fn(&cmd_view_size));
   CMD_G_STRING("view.size_not_visible",  rak::ptr_fn(&cmd_view_size_not_visible));
+  CMD_G_STRING("view.persistent",        rak::ptr_fn(&cmd_view_persistent));
+
   CMD_D_STRING("view.filter_download",   rak::ptr_fn(&cmd_view_filter_download));
   CMD_D_STRING("view.set_visible",       rak::ptr_fn(&cmd_view_set_visible));
   CMD_D_STRING("view.set_not_visible",   rak::ptr_fn(&cmd_view_set_not_visible));
@@ -450,7 +475,8 @@ initialize_command_ui() {
   // Commands that affect the default rtorrent UI.
 
 //   ADD_ANY_NONE("ui.focus",              rak::ptr_fn(&cmd_ui_focus));
-  CMD_D_ANY("ui.unfocus_download",     rak::ptr_fn(&cmd_ui_unfocus_download));
+  CMD_D_ANY("ui.unfocus_download",      rak::ptr_fn(&cmd_ui_unfocus_download));
+  CMD_N_STRING("ui.current_view.set",   rak::ptr_fn(&cmd_ui_set_view));
 
   // Move.
 
