@@ -257,22 +257,11 @@ DownloadFactory::receive_success() {
     if (m_manager->download_list()->find(infohash) == m_manager->download_list()->end())
       throw torrent::input_error("The newly created download was removed.");
 
-    // When a download scheduler is implemented, this is handled by
-    // the above insertion into download list.
-    if (m_session) {
-      // This torrent was queued for hashing or hashing when the
-      // session file was saved. Or it was in a started state.
-      if (rpc::call_command_value("d.get_hashing", rpc::make_target(download)) != Download::variable_hashing_stopped ||
-          rpc::call_command_value("d.get_state", rpc::make_target(download)) != 0)
-        m_manager->download_list()->resume(download);
+    if (!m_session)
+       rpc::call_command("d.set_state", (int64_t)m_start, rpc::make_target(download));
 
-    } else {
-      // Use the state thingie here, move below.
-      if (m_start)
-        rpc::parse_command_single(rpc::make_target(download), "d.start=");
-
-      m_manager->download_store()->save(download);
-    }
+    rpc::commands.call_catch(m_session ? "event.download.inserted_session" : "event.download.inserted_new",
+                             rpc::make_target(download), torrent::Object(), "Download event action failed: ");
 
   } catch (torrent::input_error& e) {
     std::string msg = "Command on torrent creation failed: " + std::string(e.what());
@@ -348,6 +337,7 @@ DownloadFactory::initialize_rtorrent(Download* download, torrent::Object* rtorre
                                                              download->download()->file_list()->size_chunks()));
 
   rtorrent->insert_preserve_copy("ignore_commands", (int64_t)0);
+  rtorrent->insert_preserve_copy("views", torrent::Object::create_list());
 
   rtorrent->insert_preserve_type("connection_leech", m_variables["connection_leech"]);
   rtorrent->insert_preserve_type("connection_seed", m_variables["connection_seed"]);
