@@ -165,6 +165,7 @@ Manager::Manager() :
   m_downloadList    = new DownloadList();
   m_fileStatusCache = new FileStatusCache();
   m_httpQueue       = new HttpQueue();
+  m_httpStack       = new CurlStack();
 }
 
 Manager::~Manager() {
@@ -205,16 +206,11 @@ Manager::initialize_first() {
 // Most of this should be possible to move out.
 void
 Manager::initialize_second() {
-  torrent::Http::set_factory(sigc::mem_fun(m_pollManager->get_http_stack(), &CurlStack::new_object));
-  m_httpQueue->slot_factory(sigc::mem_fun(m_pollManager->get_http_stack(), &CurlStack::new_object));
+  torrent::Http::set_factory(sigc::mem_fun(m_httpStack, &CurlStack::new_object));
+  m_httpQueue->slot_factory(sigc::mem_fun(m_httpStack, &CurlStack::new_object));
 
   CurlStack::global_init();
 
-  // Register slots to be called when a download is inserted/erased,
-  // opened or closed.
-  m_downloadList->slot_map_insert()["1_connect_logs"] = "d.initialize_logs=";
-  m_downloadList->slot_map_erase()["9_delete_tied"]   = "d.delete_tied=";
-
   torrent::connection_manager()->set_signal_handshake_log(sigc::mem_fun(this, &Manager::handshake_log));
 }
 
@@ -229,6 +225,8 @@ Manager::cleanup() {
   // here before the torrent::* objects are deleted.
 
   torrent::cleanup();
+
+  delete m_httpStack;
   CurlStack::global_cleanup();
 
   delete m_pollManager;
@@ -304,7 +302,7 @@ Manager::set_bind_address(const std::string& addr) {
       torrent::connection_manager()->set_bind_address(ai->address()->c_sockaddr());
     }
 
-    m_pollManager->get_http_stack()->set_bind_address(!ai->address()->is_address_any() ? ai->address()->address_str() : std::string());
+    m_httpStack->set_bind_address(!ai->address()->is_address_any() ? ai->address()->address_str() : std::string());
 
     rak::address_info::free_address_info(ai);
 
@@ -383,7 +381,8 @@ Manager::receive_http_failed(std::string msg) {
 void
 Manager::try_create_download(const std::string& uri, int flags, const command_list_type& commands) {
   // If the path was attempted loaded before, skip it.
-  if (!(flags & create_raw_data) &&
+  if ((flags & create_tied) &&
+      !(flags & create_raw_data) &&
       !is_network_uri(uri) &&
       !file_status_cache()->insert(uri, 0))
     return;
